from .settings import *


class Menu:

    selected_index = 0

    def __init__(
        self,
        menu_items: list[str] = ["First thing", "Second Thing", "Third Thing"],
        font=pg.font.SysFont("verdana", 20),
        color="white",
        gap=50,
        topleft=(50, 250),
        selected_background_size=(150, 50),
        selected_background_color=(0, 0, 0),
        selected_background_alpha=100,
    ) -> None:
        self.font = font
        self.color = color
        self.gap = gap
        self.topleft = topleft
        self.menu_items = menu_items
        self.length = len(self.menu_items)
        self.menu_item_images = [
            self.font.render(menu_item, True, self.color)
            for menu_item in self.menu_items
        ]
        self.menu_item_rects = [
            image.get_rect(topleft=(topleft[0], topleft[1] + gap * i))
            for i, image in enumerate(self.menu_item_images)
        ]
        self.selected_image = pg.Surface(selected_background_size)
        self.selected_image.fill(selected_background_color)
        self.selected_image.set_alpha(selected_background_alpha)
        self.selected_image_rect = self.selected_image.get_rect(
            midleft=(
                self.menu_item_rects[0].midleft[0] - 10,
                self.menu_item_rects[0].midleft[1],
            )
        )

    def add_menu_item(self, menu_item: str):
        self.menu_items.append(menu_item)
        self.length += 1
        self.menu_item_images.append(self.font.render(menu_item, True, "white"))
        self.menu_item_rects.append(
            self.menu_item_images[-1].get_rect(
                topleft=(self.topleft[0], self.menu_item_rects[-1].y + self.gap)
            )
        )

    def draw(self):
        screen.blit(self.selected_image, self.selected_image_rect)
        for i in range(self.length):
            screen.blit(self.menu_item_images[i], self.menu_item_rects[i])

    def handle_event(self, event):
        if event is not None and event.type == pg.MOUSEBUTTONDOWN:
            x, y = pg.mouse.get_pos()
            for i, rect in enumerate(self.menu_item_rects):
                if rect.collidepoint(x, y):
                    Menu.selected_index = i
                    self.selected_image_rect = self.selected_image.get_rect(
                        midleft=(
                            self.menu_item_rects[Menu.selected_index].midleft[0] - 10,
                            self.menu_item_rects[Menu.selected_index].midleft[1],
                        )
                    )

    def update(self):
        self.draw()



from .settings import *


class InputBox:
    def __init__(
        self,
        center=(WIDTH / 2, HEIGHT / 2),
        width=200,
        height=50,
        text="",
        font=pg.font.SysFont("verdana", 20),
        inactive_color="lightskyblue3",
        active_color="dodgerblue2",
        cursor_color="white",
    ):
        self.inactive_color = inactive_color
        self.active_color = active_color
        self.image = pg.Surface((width, height))
        self.rect = self.image.get_rect(center=center)
        self.text = text
        self.font = font
        self.color = self.inactive_color
        self.text_image = self.font.render(text, True, self.color)
        self.text_rect = self.text_image.get_rect(
            midleft=(self.rect.left + 5, self.rect.centery)
        )
        self.active = False
        self.pressed = False
        self.counter = 0
        self.cursor_image = pg.Surface((1, 30))
        self.cursor_image.fill(cursor_color)
        self.cursor_rect = self.cursor_image.get_rect(center=self.text_rect.midright)

    def handle_event(self, event):
        if event is not None and event.type == pg.MOUSEBUTTONDOWN:
            if self.rect.collidepoint(pg.mouse.get_pos()):
                self.active = not self.active

            else:
                self.active = False

        self.color = self.active_color if self.active else self.inactive_color
        if self.active:
            if event is not None:
                if event.type == pg.KEYDOWN:
                    if event.key == pg.K_RETURN:
                        self.text = ""
                    elif event.key == pg.K_BACKSPACE:
                        self.text = self.text[:-1]
                        self.pressed = True
                    else:
                        self.text += event.unicode
                if event.type == pg.KEYUP:
                    self.pressed = False
            # if self.pressed:
            #     self.counter += 1
            # else:
            #     self.counter = 0

            self.counter = self.counter + 1 if self.pressed else 0

            if self.counter > 100:
                self.text = self.text[:-1]
                self.counter = 80

            self.text_image = self.font.render(self.text, True, self.color)
            self.text_rect = self.text_image.get_rect(topleft=self.text_rect.topleft)
            self.cursor_rect = self.cursor_image.get_rect(
                center=self.text_rect.midright
            )

    def draw(self):
        screen.blit(self.text_image, self.text_rect.topleft)
        pg.draw.rect(screen, self.color, self.rect, width=2, border_radius=5)
        if self.active:
            screen.blit(self.cursor_image, self.cursor_rect.topleft)

    def update(self):
        self.draw()



from .settings import *


class Slider:

    busy = False

    def __init__(
        self,
        center=(WIDTH / 2, HEIGHT / 2),
        min=0,
        max=100,
        start_value=None,
        width=250,
        height=5,
        color=(255, 255, 255),
        filled_color=(0, 128, 128),
        show_value=False,
        show_label=False,
        label_text: str = "Placeholder",
        font=pg.font.SysFont("verdana", 16),
        font_color=(255, 255, 255),
    ) -> None:
        self.width = width
        self.height = height
        self.color = color
        self.filled_color = filled_color
        self.image = pg.Surface((self.width, self.height))
        self.image.fill(self.color)
        self.rect = self.image.get_rect(center=center)
        self.max = max
        self.min = min
        self.range = self.max - self.min

        self.start_value = start_value
        if self.start_value is None:
            self.start_value = self.min
        self.value = self.start_value

        self.filled_image = pg.Surface(
            ((self.value - self.min) * self.width / self.range, self.height)
        )
        self.filled_image.fill(self.filled_color)
        self.filled_rect = self.filled_image.get_rect(topleft=self.rect.topleft)
        self.pressed = False

        self.show_value = show_value
        self.show_label = show_label

        if self.show_label or self.show_value:
            self.font = font
            self.font_color = font_color

        if self.show_value:
            self.value_image = self.font.render(
                f"{round(self.value)}", True, self.font_color
            )
            self.value_rect = self.value_image.get_rect(
                midleft=(self.rect.right + 20, self.rect.centery)
            )

        if self.show_label:
            self.label_text = label_text
            self.label_image = self.font.render(self.label_text, True, self.font_color)
            self.label_rect = self.label_image.get_rect(
                midright=(self.rect.left - 20, self.rect.centery)
            )

    def draw(self):
        screen.blit(self.image, self.rect.topleft)
        screen.blit(self.filled_image, self.filled_rect.topleft)
        pg.draw.circle(
            screen,
            self.filled_color,
            (
                self.rect.left + (self.value - self.min) / self.range * self.width,
                self.rect.centery,
            ),
            8,
        )
        if self.show_value:
            screen.blit(self.value_image, self.value_rect.topleft)
        if self.show_label:
            screen.blit(self.label_image, self.label_rect.topleft)

    def update_value(self):
        x, y = pg.mouse.get_pos()
        if any(pg.mouse.get_pressed()):
            if self.rect.collidepoint(x, y) and not Slider.busy:
                Slider.busy = True
                self.pressed = True
            if self.pressed:
                self.value = (x - self.rect.left) * self.range / self.width + self.min
                if x > self.rect.right:
                    self.value = self.max
                elif x < self.rect.left:
                    self.value = self.min

            self.set_value()

        else:
            self.pressed = False
            Slider.busy = False

    def update(self):
        self.update_value()
        self.draw()

    def set_value(self, value=None):
        if value is not None:
            self.value = value
        self.filled_image = pg.Surface(
            ((self.value - self.min) * self.width / self.range, self.height)
        )
        self.filled_image.fill(self.filled_color)
        self.filled_rect = self.filled_image.get_rect(topleft=self.rect.topleft)
        if self.show_value:
            self.value_image = self.font.render(
                f"{round(self.value)}", True, self.font_color
            )
            self.value_rect = self.value_image.get_rect(
                midleft=(self.rect.right + 20, self.rect.centery)
            )

    def reset(self):
        self.set_value(self.start_value)
